import { OrthographicCamera, PerspectiveCamera, Camera as ThreeCamera } from "three";
import { EventEmitter } from "events";
import { QuickThreejs } from ".";
import { events } from "./static";
export default class Camera extends EventEmitter {
    app = new QuickThreejs();
    instance;
    miniCamera;
    updateProjectionMatrix = true;
    constructor(defaultCamera, miniCamera) {
        super();
        this._setCamera(defaultCamera);
        miniCamera && this._setMiniCamera();
        this.emit(events.CONSTRUCTED);
    }
    _setCamera(_) {
        this.removeCamera();
        if (_ === "perspective" || _ === undefined) {
            this.instance = new PerspectiveCamera(75, this.app.sizes.width / this.app.sizes.height, 0.1, 100);
            this.instance.position.z = 8;
            this.app.scene.add(this.instance);
        }
        if (_ === "orthographic") {
            this.instance = new OrthographicCamera((-this.app.sizes.aspect * this.app.sizes.frustrum) / 2, (this.app.sizes.aspect * this.app.sizes.frustrum) / 2, this.app.sizes.frustrum / 2, -this.app.sizes.frustrum / 2, -50, 50);
        }
        this.app.debug?.setCameraOrbitControl();
        this.app.debug?.setCameraHelper();
        this.instance && this.app.scene.add(this.instance);
    }
    _setMiniCamera() {
        this.removeMiniCamera();
        this.miniCamera = new PerspectiveCamera(75, this.app.sizes.width / this.app.sizes.height, 0.1, 500);
        this.miniCamera.position.z = 8;
        this.app.debug?.setMiniCameraOrbitControls();
        this.app.scene.add(this.miniCamera);
    }
    resize() {
        if (!(this.instance instanceof ThreeCamera))
            return;
        if (this.instance instanceof PerspectiveCamera)
            this.instance.aspect = this.app.sizes.width / this.app.sizes.height;
        this.instance.updateProjectionMatrix();
    }
    update() {
        if (!this.updateProjectionMatrix)
            return;
        this.instance?.updateProjectionMatrix();
        this.miniCamera?.updateProjectionMatrix();
    }
    removeCamera() {
        if (!(this.instance instanceof Camera))
            return;
        this.instance.clearViewOffset();
        this.instance.clear();
        this.instance.userData = {};
        this.app.scene.remove(this.instance);
        this.instance = undefined;
    }
    removeMiniCamera() {
        if (!(this.miniCamera instanceof Camera))
            return;
        this.miniCamera.clearViewOffset();
        this.miniCamera.clear();
        this.miniCamera.userData = {};
        this.app.scene.remove(this.miniCamera);
        this.miniCamera = undefined;
    }
    destruct() {
        this.removeCamera();
        this.removeMiniCamera();
        this.emit(events.DESTRUCTED);
        this.removeAllListeners();
    }
}
//# sourceMappingURL=Camera.js.map